/*
 * setup.cpp: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "setup.h"

// --- cMenuSetupInfosatepg
cMenuSetupInfosatepg::cMenuSetupInfosatepg(cGlobalInfosatepg *Global)
{
  global=Global;
  ChannelUseText[0]=tr("no");
  ChannelUseText[1]=tr("short text");
  ChannelUseText[2]=tr("short/long text");
  ChannelUseText[3]=tr("short text/extEPG");
  ChannelUseText[4]=tr("complete");

  newChannel = global->Channel;
  newPid = global->Pid;
  newWaitTime = global->WaitTime;
  newEventTimeDiff=int (global->EventTimeDiff/60);
  newChannelUse=NULL;
  newusePrimary=(int) global->usePrimary;

  Add(NewTitle(tr("Scan parameter")));
  Add(new cMenuEditIntItem( tr("Channel"), &newChannel,1,Channels.MaxNumber()));
  Add(new cMenuEditIntItem( tr("Pid"), &newPid,0,8191));

  Add(NewTitle(tr("Options")));
  Add(new cMenuEditIntItem( tr("WaitTime [s]"), &newWaitTime,MIN_WAITTIME,MAX_WAITTIME));
  Add(new cMenuEditIntItem( tr("Event TimeDiff [min]"), &newEventTimeDiff,
                            MIN_EVENTTIMEDIFF,MAX_EVENTTIMEDIFF));
  Add(new cMenuEditBoolItem( tr("Use Primary device"), &newusePrimary));

  if (global->InfosatChannels())
  {
    newChannelUse=new int[global->InfosatChannels()];
    Add(NewTitle(tr("Infosat channels")));
    for (int i=0; i<global->InfosatChannels(); i++)
    {
      cChannel *chan = Channels.GetByChannelID(global->GetChannelID(i));
      if (!chan) continue;
      newChannelUse[i]=global->GetChannelUse(i);
      if (newChannelUse[i]<0) newChannelUse[i]=USE_NOTHING; // to be safe!
      Add(new cMenuEditStraItem(chan->Name(),&newChannelUse[i],
           sizeof(ChannelUseText)/sizeof(const char *),ChannelUseText));
    }
  }
}

cMenuSetupInfosatepg::~cMenuSetupInfosatepg()
{
  if (newChannelUse) delete newChannelUse;
}

cOsdItem *cMenuSetupInfosatepg::NewTitle(const char *s)
{
  char str[128];
  cOsdItem *tmp;
  snprintf(str,sizeof(str),"---- %s ----", s);
  str[sizeof(str)-1]=0;
  tmp = new cOsdItem(str);
  tmp->SetSelectable(false);
  return tmp;
}

void cMenuSetupInfosatepg::Store(void)
{
  bool bReprocess=false;

  if (global->EventTimeDiff!=(60*newEventTimeDiff)) bReprocess=true;

  SetupStore("Channel", global->Channel = newChannel);
  SetupStore("Pid", global->Pid = newPid);
  SetupStore("WaitTime", global->WaitTime = newWaitTime);
  SetupStore("EventTimeDiff", newEventTimeDiff);
  global->EventTimeDiff = 60*newEventTimeDiff;
  SetupStore("usePrimary", global->usePrimary = (bool) newusePrimary);

  for (int i=0; i<global->InfosatChannels(); i++)
  {
    cChannel *chan = Channels.GetByChannelID(global->GetChannelID(i));
    if (!chan) continue;
    cString ChannelID = chan->GetChannelID().ToString();
    char *name;
    asprintf(&name,"Channel-%s",*ChannelID);
    if (!name) continue;
    if (global->SetChannelUse(i,newChannelUse[i])) bReprocess=true;
    SetupStore(name,newChannelUse[i]);
    free(name);
  }

  if (bReprocess) global->ResetProcessedFlags();
}

